package ru.dbotthepony.mc.otm.block.tech

import it.unimi.dsi.fastutil.objects.Object2ObjectOpenHashMap
import net.minecraft.Util
import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.world.item.DyeColor
import net.minecraft.world.item.context.BlockPlaceContext
import net.minecraft.world.level.BlockGetter
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.Block
import net.minecraft.world.level.block.EntityBlock
import net.minecraft.world.level.block.entity.BlockEntity
import net.minecraft.world.level.block.entity.BlockEntityTicker
import net.minecraft.world.level.block.entity.BlockEntityType
import net.minecraft.world.level.block.state.BlockState
import net.minecraft.world.level.block.state.StateDefinition
import net.minecraft.world.level.block.state.properties.EnumProperty
import net.minecraft.world.phys.shapes.CollisionContext
import net.minecraft.world.phys.shapes.VoxelShape
import ru.dbotthepony.mc.otm.block.MatteryBlock
import ru.dbotthepony.mc.otm.block.addSimpleDescription
import ru.dbotthepony.mc.otm.block.entity.tech.EnergyCounterBlockEntity
import ru.dbotthepony.mc.otm.util.asSupplier
import ru.dbotthepony.mc.otm.util.collect.SupplierMap
import ru.dbotthepony.mc.otm.registry.game.MBlockEntities
import ru.dbotthepony.mc.otm.shapes.BlockShapes
import java.util.concurrent.Callable
import java.util.function.Supplier

class EnergyCounterBlock(val color: DyeColor?) : MatteryBlock(DEFAULT_MACHINE_PROPERTIES), EntityBlock {
	init {
		tooltips.painted(color)
		addSimpleDescription()
		addSimpleDescription("2")
		addSimpleDescription("3")
		addSimpleDescription("4")
	}

	override fun newBlockEntity(blockPos: BlockPos, blockState: BlockState): BlockEntity {
		return EnergyCounterBlockEntity(blockPos, blockState)
	}

	override fun <T : BlockEntity?> getTicker(
		level: Level,
		blockState: BlockState,
		blockEntityType: BlockEntityType<T>
	): BlockEntityTicker<T>? {
		if (blockEntityType !== MBlockEntities.ENERGY_COUNTER || level.isClientSide)
			return null

		return BlockEntityTicker { _, _, _, tile -> if (tile is EnergyCounterBlockEntity) tile.tick() }
	}

	override fun getStateForPlacement(context: BlockPlaceContext): BlockState? {
		val inputDir =
			if (context.player != null && context.player!!.isCrouching) context.clickedFace.opposite else context.clickedFace

		val opposite = inputDir.opposite
		var dir: Direction? = null

		for (_dir in context.nearestLookingDirections) {
			if (_dir != inputDir && _dir != opposite) {
				dir = _dir.opposite
				break
			}
		}

		return defaultBlockState().setValue(INPUT_DIRECTION, inputDir).setValue(IF_DIRECTION, dir!!)
	}

	override fun createBlockStateDefinition(p_49915_: StateDefinition.Builder<Block, BlockState>) {
		super.createBlockStateDefinition(p_49915_)
		p_49915_.add(INPUT_DIRECTION, IF_DIRECTION)
	}

	private val shapes: Map<BlockState, VoxelShape>

	init {
		val shapes = Object2ObjectOpenHashMap<BlockState, Supplier<VoxelShape>>()

		for (state in stateDefinition.possibleStates) {
			val input: Direction = state.getValue(INPUT_DIRECTION)
			val iface: Direction = state.getValue(IF_DIRECTION)

			var shape = when (input) {
				Direction.DOWN ->  	BlockShapes.ENERGY_COUNTER_DOWN
				Direction.UP -> 	BlockShapes.ENERGY_COUNTER_UP
				Direction.NORTH -> 	BlockShapes.ENERGY_COUNTER_NORTH
				Direction.SOUTH -> 	BlockShapes.ENERGY_COUNTER_SOUTH
				Direction.WEST -> 	BlockShapes.ENERGY_COUNTER_WEST
				Direction.EAST -> 	BlockShapes.ENERGY_COUNTER_EAST
			}

			if (input === Direction.UP || input === Direction.DOWN) {
				shape = shape.rotateFromNorth(iface)
			} else if (input === Direction.EAST || input === Direction.WEST) {
				when (iface) {
					Direction.DOWN -> shape = shape.rotateAroundX(Math.PI / 2)
					Direction.UP -> shape = shape.rotateAroundX(-Math.PI / 2)
					Direction.NORTH -> { /* уже в нужном положении */ }
					Direction.SOUTH -> shape = shape.rotateAroundX(Math.PI)
					Direction.WEST -> { /* недостижимо */ }
					Direction.EAST -> { /* недостижимо */ }
				}
			} else {
				when (iface) {
					Direction.DOWN -> shape = shape.rotateAroundZ(Math.PI)
					Direction.UP -> { /* уже в нужном положении */ }
					Direction.NORTH -> { /* недостижимо */ }
					Direction.SOUTH -> { /* недостижимо */ }
					Direction.WEST -> shape = shape.rotateAroundZ(Math.PI / 2)
					Direction.EAST -> shape = shape.rotateAroundZ(-Math.PI / 2)
				}
			}

			shapes[state] = Util.backgroundExecutor().submit(Callable { shape.computeShape() }).asSupplier()
		}

		this.shapes = SupplierMap(shapes)
	}

	override fun getShape(
		blockState: BlockState,
		p_60556_: BlockGetter,
		p_60557_: BlockPos,
		p_60558_: CollisionContext
	): VoxelShape {
		return shapes[blockState] ?: super.getShape(blockState, p_60556_, p_60557_, p_60558_)
	}

	companion object {
		val INPUT_DIRECTION: EnumProperty<Direction> = EnumProperty.create("input", Direction::class.java)
		val IF_DIRECTION: EnumProperty<Direction> = EnumProperty.create("if", Direction::class.java)
	}
}
