package ru.dbotthepony.mc.otm.server.world

import net.minecraft.core.BlockPos
import net.minecraft.util.RandomSource
import net.minecraft.world.level.ChunkPos
import net.minecraft.world.level.WorldGenLevel
import net.minecraft.world.level.chunk.ChunkGenerator
import net.minecraft.world.level.levelgen.feature.FeaturePlaceContext
import net.minecraft.world.level.levelgen.feature.configurations.FeatureConfiguration
import net.minecraft.world.level.levelgen.placement.PlacementContext
import ru.dbotthepony.mc.otm.server.world.feature.EnhancedFeature
import java.util.*

class EnhancedPlacementContext {
	fun interface Placer {
		fun place(context: EnhancedPlacementContext, positions: List<PlacementPos>, feature: EnhancedFeature.Configured<*, *>)
	}

	private data class SharedState(
		val level: WorldGenLevel,
		val generator: ChunkGenerator,
		val random: RandomSource,
		val origin: ChunkPos,
		val placer: Placer,
		val vanillaContext: PlacementContext,
	)

	private val state: SharedState

	val level: WorldGenLevel
		get() = state.level
	val generator: ChunkGenerator
		get() = state.generator
	val random: RandomSource
		get() = state.random
	val origin: ChunkPos
		get() = state.origin
	private val placer: Placer
		get() = state.placer
	val vanillaContext: PlacementContext
		get() = state.vanillaContext

	val variables: PlacementVariableMap

	constructor(
		level: WorldGenLevel,
		generator: ChunkGenerator,
		random: RandomSource,
		origin: ChunkPos,
		placer: Placer,
	) {
		state = SharedState(
			level = level,
			generator = generator,
			random = random,
			origin = origin,
			placer = placer,
			vanillaContext = PlacementContext(level, generator, Optional.empty())
		)

		variables = PlacementVariableMap()
	}

	private constructor(parent: EnhancedPlacementContext) {
		this.state = parent.state
		this.variables = parent.variables.push()
	}

	private constructor(parent: EnhancedPlacementContext, context: WorldGenLevel) {
		this.state = parent.state.copy(level = context)
		this.variables = parent.variables.push()
	}

	fun push(): EnhancedPlacementContext {
		return EnhancedPlacementContext(this)
	}

	fun place(positions: List<PlacementPos>, feature: EnhancedFeature.Configured<*, *>) {
		placer.place(this, positions, feature)
	}

	fun <FC : FeatureConfiguration> vanillaFeatureContext(config: FC, position: BlockPos): FeaturePlaceContext<FC> {
		return FeaturePlaceContext(Optional.empty(), level, generator, random, position, config)
	}

	fun push(context: WorldGenLevel): EnhancedPlacementContext {
		return EnhancedPlacementContext(this, context)
	}
}
