package ru.dbotthepony.mc.otm.server.world.placement

import com.google.common.collect.ImmutableList
import com.mojang.serialization.Codec
import com.mojang.serialization.MapCodec
import net.minecraft.core.BlockPos
import net.minecraft.util.RandomSource
import net.minecraft.world.level.levelgen.placement.PlacementContext
import net.minecraft.world.level.levelgen.placement.PlacementModifier
import net.minecraft.world.level.levelgen.placement.PlacementModifierType
import ru.dbotthepony.mc.otm.registry.data.MPlacementModifiers
import java.util.stream.Stream

/**
 * Daisy-chaining placements. Required only when using placement modifiers which operate on children list
 */
class ChainPlacement(
	val children: List<PlacementModifier>
) : PlacementModifier() {
	constructor(vararg children: PlacementModifier) : this(ImmutableList.copyOf(children))

	override fun getPositions(context: PlacementContext, random: RandomSource, pos: BlockPos): Stream<BlockPos> {
		var stream = Stream.of(pos)
		children.forEach { c -> stream = stream.flatMap { c.getPositions(context, random, it) } }
		return stream
	}

	override fun type(): PlacementModifierType<*> {
		return MPlacementModifiers.CHAIN
	}

	companion object {
		val CODEC: MapCodec<ChainPlacement> = Codec.list(PlacementModifier.CODEC).xmap(::ChainPlacement, ChainPlacement::children).fieldOf("children")
	}
}
