package ru.dbotthepony.mc.otm.server.world.placement

import com.google.common.collect.ImmutableList
import com.mojang.serialization.Codec
import com.mojang.serialization.MapCodec
import ru.dbotthepony.mc.otm.server.world.EnhancedPlacementContext
import ru.dbotthepony.mc.otm.server.world.PlacementPos

/**
 * Chains placements, feeding results from one placement into next
 *
 * Each placement gets its own, new [EnhancedPlacementContext], so variables can be pushed safely into it
 */
class EnhancedChainPlacement(
	val children: List<EnhancedPlacement>
) : EnhancedPlacement {
	constructor(vararg children: EnhancedPlacement) : this(ImmutableList.copyOf(children))

	override fun evaluate(context: EnhancedPlacementContext, positions: List<PlacementPos>): List<PlacementPos> {
		var currentContext = context
		var current = positions

		children.forEach {
			current = it.evaluate(context, current)
			currentContext = currentContext.push()
		}

		return current
	}

	override val type: EnhancedPlacement.Type<*>
		get() = Companion

	companion object : EnhancedPlacement.Type<EnhancedChainPlacement> {
		override val codec: MapCodec<EnhancedChainPlacement> by lazy {
			Codec.list(EnhancedPlacement.CODEC).xmap(::EnhancedChainPlacement, EnhancedChainPlacement::children).fieldOf("children")
		}
	}
}
